@@ -8,6 +8,7 @@
 
 #include "eek/eek-types.h"
 #include "dbus.h"
+#include "panel.h"
 
 
 struct receiver;
@@ -24,7 +25,7 @@ struct rsobjects {
     struct squeek_wayland *wayland;
 };
 
-void register_ui_loop_handler(struct receiver *receiver, ServerContextService *ui, DBusHandler *dbus_handler);
+void register_ui_loop_handler(struct receiver *receiver, struct panel_manager *panel, EekboardContextService *hint_manager, DBusHandler *dbus_handler);
 
 struct rsobjects squeek_init(void);
 
