@@ -6,27 +6,32 @@
  * It's driven by the loop defined in the loop module. */
 
 use crate::animation;
+use crate::debug;
 use crate::imservice::{ ContentHint, ContentPurpose };
-use crate::main::{ Commands, PanelCommand, PixelSize };
+use crate::main::Commands;
 use crate::outputs;
-use crate::outputs::{OutputId, OutputState};
+use crate::outputs::{Millimeter, OutputId, OutputState};
+use crate::panel;
+use crate::panel::PixelSize;
+use crate::util::Rational;
 use std::cmp;
 use std::collections::HashMap;
 use std::time::Instant;
 
-#[derive(Clone, Copy)]
+
+#[derive(Clone, Copy, Debug)]
 pub enum Presence {
     Present,
     Missing,
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct InputMethodDetails {
     pub hint: ContentHint,
     pub purpose: ContentPurpose,
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub enum InputMethod {
     Active(InputMethodDetails),
     InactiveSince(Instant),
@@ -34,12 +39,13 @@ pub enum InputMethod {
 
 /// Incoming events.
 /// This contains events that cause a change to the internal state.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub enum Event {
     InputMethod(InputMethod),
     Visibility(visibility::Event),
     PhysicalKeyboard(Presence),
     Output(outputs::Event),
+    Debug(debug::Event),
     /// Event triggered because a moment in time passed.
     /// Use to animate state transitions.
     /// The value is the ideal arrival time.
@@ -59,7 +65,7 @@ impl From<outputs::Event> for Event {
 }
 
 pub mod visibility {
-    #[derive(Clone)]
+    #[derive(Clone, Debug)]
     pub enum Event {
         /// User requested the panel to show
         ForceVisible,
@@ -79,7 +85,7 @@ pub mod visibility {
 }
 
 /// The outwardly visible state.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct Outcome {
     pub visibility: animation::Outcome,
     pub im: InputMethod,
@@ -114,8 +120,8 @@ impl Outcome {
 // FIXME: handle switching outputs
         let (dbus_visible_set, panel_visibility) = match new_state.visibility {
             animation::Outcome::Visible{output, height}
-                => (Some(true), Some(PanelCommand::Show{output, height})),
-            animation::Outcome::Hidden => (Some(false), Some(PanelCommand::Hide)),
+                => (Some(true), Some(panel::Command::Show{output, height})),
+            animation::Outcome::Hidden => (Some(false), Some(panel::Command::Hide)),
         };
 
         Commands {
@@ -137,11 +143,12 @@ impl Outcome {
 /// All state changes return the next state and the optimal time for the next check.
 ///
 /// This state tracker can be driven by any event loop.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct Application {
     pub im: InputMethod,
     pub visibility_override: visibility::State,
     pub physical_keyboard: Presence,
+    pub debug_mode_enabled: bool,
     /// The output on which the panel should appear.
     /// This is stored as part of the state
     /// because it's not clear how to derive the output from the rest of the state.
@@ -163,13 +170,29 @@ impl Application {
             im: InputMethod::InactiveSince(now),
             visibility_override: visibility::State::NotForced,
             physical_keyboard: Presence::Missing,
+            debug_mode_enabled: false,
             preferred_output: None,
             outputs: Default::default(),
         }
     }
 
-    pub fn apply_event(self, event: Event, _now: Instant) -> Self {
-        match event {
+    pub fn apply_event(self, event: Event, now: Instant) -> Self {
+        if self.debug_mode_enabled {
+            println!(
+                "Received event:
+{:#?}",
+                event,
+            );
+        }
+        let state = match event {
+            Event::Debug(dbg) => Self {
+                debug_mode_enabled: match dbg {
+                    debug::Event::Enable => true,
+                    debug::Event::Disable => false,
+                },
+                ..self
+            },
+
             Event::TimeoutReached(_) => self,
 
             Event::Visibility(visibility) => Self {
@@ -235,29 +258,83 @@ impl Application {
                     ..self
                 },
             }
+        };
+
+        if state.debug_mode_enabled {
+            println!(
+                "State is now:
+{:#?}
+Outcome:
+{:#?}",
+                state,
+                state.get_outcome(now),
+            );
         }
+        state
     }
 
     fn get_preferred_height(output: &OutputState) -> Option<PixelSize> {
         output.get_pixel_size()
             .map(|px_size| {
+                // Assume isotropy.
+                // Pixels/mm.
+                let density = output.get_physical_size()
+                    .and_then(|size| size.width)
+                    .map(|width| Rational {
+                        numerator: px_size.width as i32,
+                        denominator: width.0 as u32,
+                    })
+                    // Whatever the Librem 5 has,
+                    // as a good default.
+                    .unwrap_or(Rational {
+                        numerator: 720,
+                        denominator: 65,
+                    });
+
+                // Based on what works on the L5.
+                // Exceeding that probably wastes space. Reducing makes typing harder.
+                const IDEAL_TARGET_SIZE: Rational<Millimeter> = Rational {
+                    numerator: Millimeter(948),
+                    denominator: 100,
+                };
+
+                // TODO: calculate based on selected layout
+                const ROW_COUNT: u32 = 4;
+
                 let height = {
-                    if px_size.width > px_size.height {
-                        px_size.width / 5
-                    } else {
-                        let abstract_width
-                            = PixelSize {
-                                scale_factor: output.scale as u32,
-                                pixels: px_size.width,
-                            } 
-                            .as_scaled_ceiling();
-                        if (abstract_width < 540) && (px_size.width > 0) {
-                            px_size.width * 7 / 12 // to match 360×210
+                    let ideal_height = IDEAL_TARGET_SIZE * ROW_COUNT as i32;
+                    let ideal_height_px = (ideal_height * density).ceil().0 as u32;
+
+                    // Reduce height to match what the layout can fill.
+                    // For this, we need to guess if normal or wide will be picked up.
+                    // This must match `eek_gtk_keyboard.c::get_type`.
+                    // TODO: query layout database and choose one directly
+                    let abstract_width
+                        = PixelSize {
+                            scale_factor: output.scale as u32,
+                            pixels: px_size.width,
+                        } 
+                        .as_scaled_ceiling();
+
+                    let height_as_widths = {
+                        if abstract_width < 540 {
+                            // Normal
+                            Rational {
+                                numerator: 210,
+                                denominator: 360,
+                            }
                         } else {
-                            // Here we switch to wide layout, less height needed
-                            px_size.width * 7 / 22
+                            // Wide
+                            Rational {
+                                numerator: 172,
+                                denominator: 540,
+                            }
                         }
-                    }
+                    };
+                    cmp::min(
+                        ideal_height_px,
+                        (height_as_widths * px_size.width as i32).ceil() as u32,
+                    )
                 };
                 PixelSize {
                     scale_factor: output.scale as u32,
@@ -341,7 +418,7 @@ pub mod test {
             id,
             OutputState {
                 current_mode: None,
-                transform: None,
+                geometry: None,
                 scale: 1,
             },
         );
@@ -523,4 +600,29 @@ pub mod test {
         );
 
     }
+
+    #[test]
+    fn size_l5() {
+        use crate::outputs::{Mode, Geometry, c, Size};
+        assert_eq!(
+            Application::get_preferred_height(&OutputState {
+                current_mode: Some(Mode {
+                    width: 720,
+                    height: 1440,
+                }),
+                geometry: Some(Geometry{
+                    transform: c::Transform::Normal,
+                    phys_size: Size {
+                        width: Some(Millimeter(65)),
+                        height: Some(Millimeter(130)),
+                    },
+                }),
+                scale: 2,
+            }),
+            Some(PixelSize {
+                scale_factor: 2,
+                pixels: 420,
+            }),
+        );
+    }
 }
